@@ -40,6 +40,7 @@ module Control.Monad.Logger
     , runStderrLoggingT
     , runStdoutLoggingT
     , runChanLoggingT
+    , runFileLoggingT
     , unChanLoggingT
     , withChannelLogger
     , filterLogger
@@ -73,6 +74,7 @@ module Control.Monad.Logger
     , logErrorN
     , logOtherN
     -- * Non-TH logging with source
+    , logWithoutLoc
     , logDebugNS
     , logInfoNS
     , logWarnNS
@@ -89,6 +91,7 @@ module Control.Monad.Logger
     -- * utilities for defining your own loggers
     , defaultLogStr
     , Loc (..)
+    , defaultLoc
     ) where
 
 #if WITH_TEMPLATE_HASKELL
@@ -101,7 +104,7 @@ import Control.Applicative (Applicative (..))
 import Control.Concurrent.Chan (Chan(),writeChan,readChan)
 import Control.Concurrent.STM
 import Control.Concurrent.STM.TBChan
-import Control.Exception.Lifted (onException)
+import Control.Exception.Lifted (onException, bracket)
 import Control.Monad (liftM, ap, when, void, forever)
 import Control.Monad.Base (MonadBase (liftBase))
 import Control.Monad.Loops (untilM)
@@ -143,7 +146,7 @@ import qualified Data.ByteString.Char8 as S8
 
 import Data.Monoid (mappend, mempty)
 import System.Log.FastLogger
-import System.IO (Handle, stdout, stderr)
+import System.IO (Handle, IOMode(AppendMode), BufferMode(LineBuffering), openFile, hClose, hSetBuffering, stdout, stderr)
 
 import Control.Monad.Cont.Class   ( MonadCont (..) )
 import Control.Monad.Error.Class  ( MonadError (..) )
@@ -200,8 +203,8 @@ type CharPos = (Int, Int)
 -- | A @Monad@ which has the ability to log messages in some manner.
 class Monad m => MonadLogger m where
     monadLoggerLog :: ToLogStr msg => Loc -> LogSource -> LogLevel -> msg -> m ()
-    default monadLoggerLog :: (Trans.MonadTrans t, MonadLogger (t m), ToLogStr msg)
-                           => Loc -> LogSource -> LogLevel -> msg -> t m ()
+    default monadLoggerLog :: (MonadLogger m', Trans.MonadTrans t, MonadLogger (t m'), ToLogStr msg, m ~ t m')
+                           => Loc -> LogSource -> LogLevel -> msg -> m ()
     monadLoggerLog loc src lvl msg = Trans.lift $ monadLoggerLog loc src lvl msg
 
 -- | An extension of @MonadLogger@ for the common case where the logging action
@@ -209,14 +212,14 @@ class Monad m => MonadLogger m where
 -- logging function itself can be extracted as a first-class value, which can
 -- make it easier to manipulate monad transformer stacks, as an example.
 --
--- Since 0.3.10
+-- @since 0.3.10
 class (MonadLogger m, MonadIO m) => MonadLoggerIO m where
     -- | Request the logging function itself.
     --
-    -- Since 0.3.10
+    -- @since 0.3.10
     askLoggerIO :: m (Loc -> LogSource -> LogLevel -> LogStr -> IO ())
-    default askLoggerIO :: (Trans.MonadTrans t, MonadLogger (t m), MonadIO (t m))
-                        => t m (Loc -> LogSource -> LogLevel -> LogStr -> IO ())
+    default askLoggerIO :: (Trans.MonadTrans t, MonadLoggerIO n, m ~ t n)
+                        => m (Loc -> LogSource -> LogLevel -> LogStr -> IO ())
     askLoggerIO = Trans.lift askLoggerIO
 
 
@@ -326,7 +329,7 @@ logOtherSH = logTHShow . LevelOther
 
 -- | Lift a location into an Exp.
 --
--- Since 0.3.1
+-- @since 0.3.1
 liftLoc :: Loc -> Q Exp
 liftLoc (Loc a b c (d1, d2) (e1, e2)) = [|Loc
     $(lift a)
@@ -361,15 +364,27 @@ logOtherS = [|\src level msg -> monadLoggerLog $(qLocation >>= liftLoc) src (Lev
 
 -- | Monad transformer that disables logging.
 --
--- Since 0.2.4
+-- @since 0.2.4
 newtype NoLoggingT m a = NoLoggingT { runNoLoggingT :: m a }
 
+#if __GLASGOW_HASKELL__ < 710
 instance Monad m => Functor (NoLoggingT m) where
     fmap = liftM
 
 instance Monad m => Applicative (NoLoggingT m) where
     pure = return
     (<*>) = ap
+#else
+instance Functor m => Functor (NoLoggingT m) where
+    fmap f = NoLoggingT . fmap f . runNoLoggingT
+    {-# INLINE fmap #-}
+
+instance Applicative m => Applicative (NoLoggingT m) where
+    pure = NoLoggingT . pure
+    {-# INLINE pure #-}
+    f <*> a = NoLoggingT (runNoLoggingT f <*> runNoLoggingT a)
+    {-# INLINE (<*>) #-}
+#endif
 
 instance Monad m => Monad (NoLoggingT m) where
     return = NoLoggingT . return
@@ -449,15 +464,29 @@ instance MonadIO m => MonadLoggerIO (NoLoggingT m) where
 
 -- | Monad transformer that adds a new logging function.
 --
--- Since 0.2.2
+-- @since 0.2.2
 newtype LoggingT m a = LoggingT { runLoggingT :: (Loc -> LogSource -> LogLevel -> LogStr -> IO ()) -> m a }
 
+#if __GLASGOW_HASKELL__ < 710
 instance Monad m => Functor (LoggingT m) where
     fmap = liftM
 
 instance Monad m => Applicative (LoggingT m) where
     pure = return
     (<*>) = ap
+#else
+instance Functor m => Functor (LoggingT m) where
+    fmap f logger = LoggingT $ \loggerFn -> fmap f $ (runLoggingT logger) loggerFn
+    {-# INLINE fmap #-}
+
+instance Applicative m => Applicative (LoggingT m) where
+    pure = LoggingT . const . pure
+    {-# INLINE pure #-}
+    loggerF <*> loggerA = LoggingT $ \loggerFn ->
+                                       (runLoggingT loggerF) loggerFn
+                                       <*> (runLoggingT loggerA) loggerFn
+    {-# INLINE (<*>) #-}
+#endif
 
 instance Monad m => Monad (LoggingT m) where
     return = LoggingT . const . return
@@ -624,15 +653,24 @@ defaultLogStrWithoutLoc loc src level msg =
 -}
 
 
+-- | Run a block using a @MonadLogger@ instance which appends to the specified file.
+--
+-- @since 0.3.22
+runFileLoggingT :: MonadBaseControl IO m => FilePath -> LoggingT m a -> m a
+runFileLoggingT fp log = bracket
+    (liftBase $ openFile fp AppendMode)
+    (liftBase . hClose)
+    $ \h -> liftBase (hSetBuffering h LineBuffering) >> (runLoggingT log) (defaultOutput h)
+
 -- | Run a block using a @MonadLogger@ instance which prints to stderr.
 --
--- Since 0.2.2
+-- @since 0.2.2
 runStderrLoggingT :: MonadIO m => LoggingT m a -> m a
 runStderrLoggingT = (`runLoggingT` defaultOutput stderr)
 
 -- | Run a block using a @MonadLogger@ instance which prints to stdout.
 --
--- Since 0.2.2
+-- @since 0.2.2
 runStdoutLoggingT :: MonadIO m => LoggingT m a -> m a
 runStdoutLoggingT = (`runLoggingT` defaultOutput stdout)
 
@@ -663,7 +701,7 @@ unChanLoggingT chan = forever $ do
 --   channel of the indicated size, and only actually log them if there is an
 --   exception.
 --
--- Since 0.3.2
+-- @since 0.3.2
 withChannelLogger :: (MonadBaseControl IO m, MonadIO m)
                   => Int         -- ^ Number of messages to keep
                   -> LoggingT m a
@@ -684,7 +722,7 @@ withChannelLogger size action = LoggingT $ \logger -> do
 --
 -- This can be a convenient way, for example, to ignore debug level messages.
 --
--- Since 0.3.13
+-- @since 0.3.13
 filterLogger :: (LogSource -> LogLevel -> Bool)
              -> LoggingT m a
              -> LoggingT m a
@@ -709,6 +747,11 @@ instance MonadReader r m => MonadReader r (LoggingT m) where
   ask = Trans.lift ask
   local = mapLoggingT . local
 
+-- | @since 0.3.24
+instance MonadReader r m => MonadReader r (NoLoggingT m) where
+  ask = Trans.lift ask
+  local = mapNoLoggingT . local
+
 mapLoggingT :: (m a -> n b) -> LoggingT m a -> LoggingT n b
 mapLoggingT f = LoggingT . (f .) . runLoggingT
 
@@ -733,6 +776,9 @@ instance MonadWriter w m => MonadWriter w (NoLoggingT m) where
     listen = mapNoLoggingT listen
     pass   = mapNoLoggingT pass
 
+-- | dummy location, used with 'logWithoutLoc'
+--
+-- @since 0.3.23
 defaultLoc :: Loc
 defaultLoc = Loc "<unknown>" "<unknown>" "<unknown>" (0,0) (0,0)
 
@@ -740,6 +786,9 @@ isDefaultLoc :: Loc -> Bool
 isDefaultLoc (Loc "<unknown>" "<unknown>" "<unknown>" (0,0) (0,0)) = True
 isDefaultLoc _ = False
 
+-- |
+--
+-- @since 0.3.23
 logWithoutLoc :: (MonadLogger m, ToLogStr msg) => LogSource -> LogLevel -> msg -> m ()
 logWithoutLoc = monadLoggerLog defaultLoc
 
